/************************************************************************************
This code is designed by Mingdeyang Science and Technology Education.

We hope to help students understand the IC /FPGA code better and develop good programming habits through strict standards and planning.Its copyright is protected.

It is only used for personal study and can not be used for commercial or for-profit purposes without permission

For  more  FPGA  learning  materials,  see  the  forum: ( http://fpgabbs.com/)  and  the  official  website:(http://www.mdy-edu.com)
*************************************************************************************/
module  traf_light1(
    clk             ,
    rst_n           ,
    led_east        ,
    led_south       ,
    led_west        ,
    led_north       
    );


parameter   LED_LEN         =   3;
parameter   COUNT_1S        =   26'd5000_0000;
parameter   COUNT_WID       =   28;

input                   clk      ;
input                   rst_n    ;
output  [LED_LEN-1:0]   led_east ;
output  [LED_LEN-1:0]   led_south;
output  [LED_LEN-1:0]   led_west ;
output  [LED_LEN-1:0]   led_north;

reg     [LED_LEN-1:0]   led_east ;
reg     [LED_LEN-1:0]   led_south;
reg     [LED_LEN-1:0]   led_west ;
reg     [LED_LEN-1:0]   led_north;

reg	    [COUNT_WID-1:0] e_counter;        
reg     [COUNT_WID-1:0] s_counter;
reg     [COUNT_WID-1:0] w_counter;
reg     [COUNT_WID-1:0] n_counter;

wire                    add_e_counter;
wire                    end_e_counter;
wire                    add_w_counter;
wire                    end_w_counter;
wire                    add_s_counter;
wire                    end_s_counter;
wire                    add_n_counter;
wire                    end_n_counter;

always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        e_counter <= 0; 
    end
    else if(add_e_counter) begin
        if(end_e_counter)
            e_counter <= 0; 
        else
            e_counter <= e_counter+1 ;
   end
end
assign add_e_counter = 1;
assign end_e_counter = add_e_counter  && e_counter == COUNT_1S-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        w_counter <= 0; 
    end
    else if(add_w_counter) begin
        if(end_w_counter)
            w_counter <= 0; 
        else
            w_counter <= w_counter+1 ;
   end
end
assign add_w_counter = 1;
assign end_w_counter = add_w_counter  && w_counter == 2*COUNT_1S-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        s_counter <= 0; 
    end
    else if(add_s_counter) begin
        if(end_s_counter)
            s_counter <= 0; 
        else
            s_counter <= s_counter+1 ;
   end
end
assign add_s_counter = 1;
assign end_s_counter = add_s_counter  && s_counter == 3*COUNT_1S-1 ;


always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        n_counter <= 0; 
    end
    else if(add_n_counter) begin
        if(end_n_counter)
            n_counter <= 0; 
        else
            n_counter <= n_counter+1 ;
   end
end
assign add_n_counter = 1;
assign end_n_counter = add_n_counter  && n_counter == 4*COUNT_1S-1 ;


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_east<={{(LED_LEN-1){1'b1}},1'b0};
    end
    else if(end_e_counter)begin
        led_east<={led_east[LED_LEN-2:0],led_east[LED_LEN-1]};
    end
    else begin
        led_east<=led_east;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_west<={{(LED_LEN-1){1'b1}},1'b0};
    end
    else if(end_w_counter)begin
        led_west<={led_west[LED_LEN-2:0],led_west[LED_LEN-1]};
    end
    else begin
        led_west<=led_west;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_south<={{(LED_LEN-1){1'b1}},1'b0};
    end
    else if(end_s_counter)begin
        led_south<={led_south[LED_LEN-2:0],led_south[LED_LEN-1]};
    end
    else begin
        led_south<=led_south;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        led_north<={{(LED_LEN-1){1'b1}},1'b0};
    end
    else if(end_n_counter)begin
        led_north<={led_north[LED_LEN-2:0],led_north[LED_LEN-1]};
    end
    else begin
        led_north<=led_north;
    end
end

endmodule
